open System.IO
open System.Xml

type FileType =
  | Android
  | IOS
  | Shared

let inline addAttr (doc: XmlDocument) (node: XmlNode) name value =
  let attr = doc.CreateAttribute(name)
  attr.Value <- value
  
  node.Attributes.Append(attr) |> ignore

let inline tryElementByTagOrDefault (doc: XmlDocument) tag def =
  let v = doc.GetElementsByTagName(tag)
  if v.Count = 0 then def else (v.ItemOf 0).InnerText

let inline createNode (doc: XmlDocument) (tp: XmlNodeType) (name: string) =
  doc.CreateNode(tp, name, "")

let inline createElementAndAppend (doc: XmlDocument) (toNode: XmlNode) (name: string) (value: string) =
  let node = createNode doc XmlNodeType.Element name
  node.InnerText <- value
  toNode.AppendChild node |> ignore

for file in Directory.EnumerateFiles("./", "*.fsproj", SearchOption.AllDirectories) do
  printfn "Processing file %A" file
  
  let file = Path.GetFullPath file
  
  if File.ReadAllText(file).Contains("<Project Sdk=\"Microsoft.NET.Sdk\">") then
    ()
  else
    let outFile = file // file.Replace("fsproj", "fproj")
    
    let fileType =
      if file.Contains "Android" then FileType.Android
      elif file.Contains "iOS" then FileType.IOS
      else FileType.Shared
    
    let targetFramework =
      match fileType with
      | FileType.Android -> "net6.0-android"
      | FileType.IOS -> "net6.0-ios13.0"
      | FileType.Shared -> "net6.0"
    
    let doc = XmlDocument()
    
    printfn "Loading document..."
    doc.Load(file)
    
    let outputType = tryElementByTagOrDefault doc "OutputType" "Library"
    let rootNamespace = tryElementByTagOrDefault doc "RootNamespace" ""
    let assemblyName = tryElementByTagOrDefault doc "AssemblyName" rootNamespace

    printfn "Document is loaded!"

    let newDoc = XmlDocument()
    let addAttr = addAttr newDoc
    let createNode = createNode newDoc
    
    let declaration = newDoc.CreateXmlDeclaration("1.0", "utf-8", "")
    newDoc.AppendChild(declaration) |> ignore
    
    let project = createNode XmlNodeType.Element "Project"

    addAttr project "Sdk" "Microsoft.NET.Sdk"
    
    let mainGroup = createNode XmlNodeType.Element "PropertyGroup"
    project.AppendChild(mainGroup) |> ignore
    
    newDoc.AppendChild(project) |> ignore

    createElementAndAppend newDoc mainGroup "TargetFramework" targetFramework
    createElementAndAppend newDoc mainGroup "OutputType" outputType
    createElementAndAppend newDoc mainGroup "RootNamespace" rootNamespace
    createElementAndAppend newDoc mainGroup "AssemblyName" assemblyName
    
    for item in doc.GetElementsByTagName("ItemGroup") do
      let mutable add = false
      for node in item.ChildNodes do
        if node.Name = "Compile" || node.Name = "None" || node.Name = "BundleResource" then
          add <- true
      
      if add then
        let item = newDoc.ImportNode(item, true)
        project.AppendChild(item) |> ignore
    
    let refsGroup = createNode XmlNodeType.Element "ItemGroup"
    project.AppendChild(refsGroup) |> ignore
    
    
    for item in doc.GetElementsByTagName("ProjectReference") do
      for attr in item.Attributes do
        if attr.Name = "Include" then
          let node = createNode XmlNodeType.Element "ProjectReference"
          addAttr node "Include" attr.Value
          refsGroup.AppendChild(node) |> ignore
    
    let node = createNode XmlNodeType.Element "Import"
    addAttr node "Project" "..\..\.paket\Paket.Restore.targets"
    project.AppendChild(node) |> ignore
    
    newDoc.Save(outFile)

    let newText = File.ReadAllText(outFile).Replace(" xmlns=\"http://schemas.microsoft.com/developer/msbuild/2003\"", "")
    File.WriteAllText(outFile, newText)